/**
 * Класс для отрисовки компонента
 */
function FavoritesComponent() {
    var $visualElement = null;
    var $emptyDiv = null;
    var $favoritesDiv = null;
    var favorites;
    var shopConfig;


    /**
     * "Отрисовка" компонента.
     *
     * @param {object} selector - Объект-контейнер, куда будут добавляться элементы.
     * @param {string} config - Конфиг для заполнения формы в формате JSON.
     * @param {int} instanceId - id инстанса компонента.
     */
    this.renderComponent = function(selector, config, instanceId) {
        createVisualElement();
        selector.append($visualElement);
        getShopConfig();
    };

    /**
     * Создание основного визуального элемента истории заказов
     */
    function createVisualElement() {
        var template = '<div class="page ui_text_color page-on-center">' +
                            '<div class="page-content favorites">' +
                                '<div id="favoritesItems">' +
                                    '<div id="favoritesDiv" class="category-list"></div>' +
                                    '<div id="emptyDiv" class="category-list" style="display: none">' +
                                        '<div style="padding-left: 25px" class="">' +
                                            l10n.favorites[lang].favorites_empty +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>';


        $visualElement = $(template);
        $favoritesDiv = $visualElement.find('#favoritesDiv');
        $emptyDiv = $visualElement.find('#emptyDiv');
    }

    /**
     * Запрос избранных товаров с сервера
     */
    function getFavorites() {
        if(shopInstance === null || shopInstance === undefined) {
            refreshData('[]');
            return;
        }

        favorites = $.parseJSON(localStorage.getItem(globalDeviceId + '---' + shopInstance.id));

        if(!Array.isArray(favorites) || favorites.length == 0) {
            refreshData('[]');
            return;
        }

        $.get(schema + '://' + serverUrl + '/shop/get-products',
            {
                instance_id: shopInstance.id,
                device_id: globalDeviceId,
                product_ids: favorites,
            },
            function(data) {
                refreshData(data);
            })
            .fail(function(xhr, status, error) {
                myApp.alert(status);
            });
    }

    /**
     * Запрос конфиг магазина
     */
    function getShopConfig() {
        if(shopInstance === null || shopInstance === undefined) {
            refreshData('[]');
            return;
        }

        $.get(schema + '://' + serverUrl + '/site/get-config',
            {
                instance_id: shopInstance.id,
            },
            function(data) {
                shopConfig = $.parseJSON(data);
                getFavorites();
            })
            .fail(function(xhr, status, error) {
                myApp.alert(status);
            });
    }

    /**
     * Отображение списка избранных товаров
     *
     * @param {string} data - данные в JSON формате.
     */
    function refreshData(data) {
        var products = $.parseJSON(data);
        $favoritesDiv.empty();
        $emptyDiv.hide();

        for(var i = 0; i < products.length; i++) {
            $favoritesDiv.append(createProductCard(products[i]));
        }

        if(products.length === 0) {
            $emptyDiv.show();
        }
    }

    /**
     * Создать HTML представление товара.
     *
     * @param {object} item - данные товара.
     * @return {object} jQuery объект - карточка заказа.
     */
    function createProductCard(item) {
        var template = '<div class="favorites-item">' +
                            '<div class="row no-gutter favorites-item-container">' +
                                '<div class="favorites-item-info col-33">' +
                                    '<div class="favorites-item-img-div">' +
                                        '<div class="favorites-item-img"></div>' +
                                    '</div>' +
                                '</div>' +
                                '<div class="favorites-item-info col-50">' +
                                    '<div class="catalog_item">' +
                                        item.name +
                                    '</div>' +
                                    '<div class="catalog_price">' +
                                        '<span id="itemPrice">' +
                                            '{0} {1}'.format(item.price, getLocalizedCurrency(shopConfig.storeCurrency)) +
                                        '</span>' +
                                    '</div>' +
                                    '<a href="#" id="btnStore" class="button ui_button_text ui_button_background">' +
                                        '{0}'.format(l10n.favorites[lang].to_store) +
                                    '</a>' +
                                '</div>' +
                                '<div class="favorites-item-info col-10">' +
                                    '<a href="#" id="btnRemove" class="link icon-only">' +
                                        '<i class="fa fa-trash ui_text_color" style="font-size:24px"></i>' +
                                    '</a>' +
                                '</div>' +
                            '</div>' +
                        '</div>';

        var $card = $(template);
        $card.data('productId', item.id);
        $card.find('#btnRemove').click({productId: item.id}, removeProduct);

        var $link = $card.find('#btnStore');
        $link.prop('href', shopInstance.href + '?productId=' + item.id);
        $link.attr('data-context',
            '{"Title": "' + l10n.favorites[lang].shop_name + '", ' +
            '"IsBasketVisible": true}');
        setImage($card, item);

        if(!shopConfig.settings.showPrices) {
            $card.find('#itemPrice').css('display', 'none');
        }

        return $card;
    }


    /**
     * Взять локализованую валюту
     *
     * @param {string} currencyIso - название валюты в ISO стандарте.
     * @return {string} локализованное название валюты.
     */
    function getLocalizedCurrency(currencyIso) {
        var currency = l10n.favorites[lang][currencyIso];

        if(currency === undefined) {
            return currencyIso;
        }

        return currency;
    }


    /**
     * Установить изображение товара
     *
     * @param {object} $card - jQuery объект - карточка товара.
     * @param {object} product - данные товара.
     */
    function setImage($card, product) {
        var images = [];

        if(product.image !== undefined && product.image !== null && product.image !== '') {
            images = $.parseJSON(product.image);
        }

        if(images.length > 0) {
            var $imageDiv = $card.find('.favorites-item-img');

            $imageDiv.css('background-image', 'url("{0}")'.format(images[0]));
        }
    }

    /**
     * Удаление товара из избранного
     *
     * @param {object} event - event объект события.
     */
    function removeProduct(event) {
        var productId = event.data.productId;

        var index = $.inArray(productId, favorites);

        if(index > -1) {
            favorites.splice(index, 1);
        }

        localStorage.setItem(globalDeviceId + '---' + shopInstance.id, JSON.stringify(favorites));

        removeVisualProduct(productId);
    }

    /**
     * Удаление визуального элемента товара из избранного
     *
     * @param {int} productId -  id товара
     */
    function removeVisualProduct(productId) {
        var $div = findProductDiv(productId);
        $div.remove();
    }

    /**
     * Находит элемент избранного по id товара
     *
     * @param {int} productId -  id товара
     * @return {object} jQuery представление товара
     */
    function findProductDiv(productId) {
        var divs = $visualElement.find('.favorites-item');

        for(var i = 0; i < divs.length; i++) {
            var $div = $(divs[i]);

            if($div.data('productId') === productId) {
                return $div;
            }
        }

        return null;
    }
}
